package com.qmth.distributed.print.business.templete.execute;

import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.TBTask;
import com.qmth.distributed.print.business.enums.TaskResultEnum;
import com.qmth.distributed.print.business.enums.TaskStatusEnum;
import com.qmth.distributed.print.business.service.ExamDetailService;
import com.qmth.distributed.print.business.templete.export.AsyncExportTaskTemplete;
import com.qmth.distributed.print.business.util.ExcelUtil;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.util.List;
import java.util.Map;
import java.util.StringJoiner;

/**
 * @Description: 异步考务数据导出
 * @Author: CaoZixuan
 * @Date: 2021-04-10
 */
@Service
public class AsyncExaminationExportTemplateService extends AsyncExportTaskTemplete {
    @Resource
    private ExamDetailService examDetailService;

    public static final String OBJ_TITLE = "考务数据";
    private final static Logger log = LoggerFactory.getLogger(AsyncExaminationExportTemplateService.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result exportTask(Map<String, Object> map, Long printPlanId, String courseCode, String paperNumber, String examPlace, String examRoom, String packageCode) throws Exception {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        Long userId = tbTask.getCreateId();
        Long schoolId = tbTask.getSchoolId();

        StringJoiner stringJoinerSummary = new StringJoiner("\n")
                .add(MessageFormat.format("{0}{1}{2}", FORMAT_TIME, BEGIN_TITLE, OBJ_TITLE));
        tbTask.setStatus(TaskStatusEnum.RUNNING);

        List<ExaminationResult> examinationResultList = examDetailService.findExaminationBriefPage(schoolId,
                printPlanId, courseCode, paperNumber, examPlace, examRoom, packageCode,
                SystemConstant.PAGE_NUMBER, SystemConstant.PAGE_SIZE).getRecords();
        try {
            ExcelUtil.excelExport("考务信息查询", ExaminationResult.class, examinationResultList, ServletUtil.getResponse());
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", FORMAT_TIME, FINISH_TITLE, 111, FINISH_SIZE));
            tbTask.setResult(TaskResultEnum.SUCCESS);
        } catch (Exception e) {
            log.error("请求出错", e);
            stringJoinerSummary.add(MessageFormat.format("{0}{1}{2}{3}", FORMAT_TIME, EXCEPTION_TITLE, EXCEPTION_DATA, e.getMessage()));
            tbTask.setResult(TaskResultEnum.ERROR);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {//生成txt文件
            tbTask.setSummary(stringJoinerSummary.toString());
        }
        return ResultUtil.ok(map);
    }
}
