package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.DrawRuleEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.*;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Base64Util;
import com.qmth.distributed.print.common.util.HexUtils;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    ExamDetailCourseService detailCourseService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardService examCardService;

    @Resource
    ExamCardDetailService examCardDetailService;

    @Resource
    ExamStudentService examStudentService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    OssUtil ossUtil;

    @Resource
    CacheService cacheService;

    @Resource
    FreemarkerUtil freemarkerUtil;

    /**
     * 执行导入用户逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Gson gson = new Gson();
        LinkedMultiValueMap<Long, SysUser> tbUserLinkedMultiValueMap = new LinkedMultiValueMap<>();
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> userImportDtoList = excelMap.get(i);
            for (int y = 0; y < userImportDtoList.size(); y++) {
                UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                SysUser sysUser = gson.fromJson(gson.toJson(userImportDto), SysUser.class);

                //查用户登录名是否重复
                QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
                sysUserQueryWrapper.lambda().eq(SysUser::getLoginName, userImportDto.getLoginName());
                long count = sysUserService.count(sysUserQueryWrapper);
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]重复")));
                }
                //查学校是否存在
                if (Objects.nonNull(userImportDto.getSchoolCode())) {
                    QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
                    basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, userImportDto.getSchoolCode());
                    BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
                    if (Objects.isNull(basicSchool)) {
                        throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]不存在")));
                    }
                    sysUser.setSchoolId(basicSchool.getId());
                }
                sysUser.setInsertInfo(requestUser.getId());
                sysUser.setPassword(Base64Util.encode(userImportDto.getPassword().getBytes()));
                tbUserLinkedMultiValueMap.add(userImportDto.getSysRole().getId(), sysUser);
            }
        }
        tbUserLinkedMultiValueMap.forEach((k, v) -> {
            sysUserService.saveBatch(v);
            v.forEach(s -> {
                commonService.addUserRolePrivilege(s, k);
            });
        });
        return map;
    }

    /**
     * 创建pdf逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeCreatePdfLogic(Map<String, Object> map) throws IOException {
        List<BasicAttachment> basicAttachmentList = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            //这里为保存附件html的逻辑
            //查询题卡详情是否有附件id，有的话则把以前的附件删除
            Set<Long> attachmentIds = null;

            //查询printPlan
            ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
            if (Objects.isNull(examPrintPlan)) {
                throw ExceptionResultEnum.ERROR.exception("印刷计划为空");
            }

            BasicSchool basicSchool = cacheService.schoolCache(examPrintPlan.getSchoolId());

            //查询examDetail
            QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
            examDetailQueryWrapper.lambda().eq(ExamDetail::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamDetail::getPrintPlanId, tbTask.getPrintPlanId());
            List<ExamDetail> examDetailList = detailService.list(examDetailQueryWrapper);
            if (Objects.isNull(examDetailList) || examDetailList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("考务计划为空");
            }
            attachmentIds = Objects.isNull(attachmentIds) ? attachmentIds = new HashSet<>() : attachmentIds;
            for (ExamDetail examDetail : examDetailList) {
                if (Objects.nonNull(examDetail.getAttachmentId())) {
                    attachmentIds.add(examDetail.getAttachmentId());
                }

                //查询examDetailCourse
                QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
                examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
                List<ExamDetailCourse> examDetailCourseList = detailCourseService.list(examDetailCourseQueryWrapper);

                List<String> variablePdfList = new ArrayList<>();//变量印品(签到表、卷袋贴)
                List<String> ordinaryPdfList = new ArrayList<>();//普通印品(登记表)
                List<String> paperPdfList = new ArrayList<>();//所有试卷
                List<String> examStudentPdfList = new ArrayList<>();//所有题卡
                List<String> backupPaperPdfList = new ArrayList<>();//备份试卷
                List<String> cardPdfList = new ArrayList<>();//备份题卡

                if (Objects.nonNull(examPrintPlan.getOrdinaryContent())) {
                    //获取普通印品
                    JSONArray jsonArrayOrdinary = JSONArray.parseArray(examPrintPlan.getOrdinaryContent());
                }

                for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                    //查询试卷
                    QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
                    examTaskQueryWrapper.lambda().eq(ExamTask::getSchoolId, sysUser.getSchoolId())
                            .eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode())
                            .eq(ExamTask::getCourseName, examDetailCourse.getCourseName())
                            .eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber())
                            .eq(ExamTask::getEnable, true)
                            .eq(ExamTask::getStatus, ExamStatusEnum.FINISH);
                    List<ExamTask> examTaskList = examTaskService.list(examTaskQueryWrapper);
                    if (Objects.isNull(examTaskList) || examTaskList.size() == 0) {
                        throw ExceptionResultEnum.ERROR.exception("命题任务为空");
                    }
                    Set<Long> examTaskIds = examTaskList.stream().map(s -> s.getId()).collect(Collectors.toSet());

                    QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
                    examTaskDetailQueryWrapper.lambda().in(ExamTaskDetail::getExamTaskId, examTaskIds)
                            .eq(ExamTaskDetail::getEnable, true);
                    List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(examTaskDetailQueryWrapper);
                    JSONObject jsonObject = new JSONObject();
                    JSONArray jsonArray = new JSONArray();
                    for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                        //查询题卡
                        ExamCard examCard = examCardService.getById(examTaskDetail.getCardId());
                        if (Objects.isNull(examCard)) {
                            throw ExceptionResultEnum.ERROR.exception("题卡为空");
                        }
                        QueryWrapper<ExamCardDetail> examCardDetailQueryWrapper = new QueryWrapper<>();
                        examCardDetailQueryWrapper.lambda().eq(ExamCardDetail::getCardId, examCard.getId());
                        List<ExamCardDetail> examCardDetailList = examCardDetailService.list(examCardDetailQueryWrapper);

                        //抽取卷型
                        String paperType = getPaperType(examPrintPlan, examTaskDetail);

                        //获取试卷pdf
                        getPaperPdf(examTaskDetail, examPrintPlan.getBackupCount(), paperPdfList, backupPaperPdfList);

                        basicAttachmentList = Objects.isNull(basicAttachmentList) ? basicAttachmentList = new ArrayList<>() : basicAttachmentList;
                        for (ExamCardDetail examCardDetail : examCardDetailList) {
                            getCardAttachmentId(examCardDetail, attachmentIds);

                            //把模板页面上的 ${} 替换成实际内容
                            String cardContent = replaceHtmlTemplete(examCardDetail);
                            String studentContent = cardContent;

                            for (int i = 1; i <= examPrintPlan.getBackupCount(); i++) {
                                basicAttachmentList.add(cardHtml(cardContent, paperType, examDetailCourse, examCard, jsonArray, sysUser.getId(), cardPdfList));
                            }

                            //查询考生
                            QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
                            examStudentQueryWrapper.lambda().eq(ExamStudent::getSchoolId, sysUser.getSchoolId())
                                    .eq(ExamStudent::getExamDetailCourseId, examTaskDetail.getId());
                            List<ExamStudent> examStudentList = examStudentService.list(examStudentQueryWrapper);

                            Integer signVariableBackupCount = null, packagVariableBackupCount = null;
                            if (Objects.nonNull(examPrintPlan.getVariableContent())) {
                                //获取变量印品
                                JSONArray jsonArrayVariable = JSONArray.parseArray(examPrintPlan.getVariableContent());
                                for (int i = 0; i < jsonArrayVariable.size(); i++) {
                                    JSONObject jsonObjectVariable = jsonArrayVariable.getJSONObject(i);
                                    String type = (String) jsonObjectVariable.get("type");
                                    Long attachmentId = (Long) jsonObjectVariable.get("attachmentId");
                                    BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                                    String htmlContent = new String(commonService.getFileByte(basicAttachment.getPath(), false), StandardCharsets.UTF_8);
                                    log.info("htmlContent:{}", htmlContent);
                                    if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "SIGN")) {//签到表
                                        signVariableBackupCount = (Integer) jsonObjectVariable.get("backupCount");
                                    } else if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "PACKAG")) {//卷袋贴
                                        packagVariableBackupCount = (Integer) jsonObjectVariable.get("backupCount");
                                    }
//                                    createSignBook(basicAttachment, htmlContent, examDetailCourse, basicSchool.getName(), examStudentList, variablePdfList);
                                    createSignBook(basicAttachment, examDetailCourse, basicSchool.getName(), examStudentList, variablePdfList);
                                }
                            }

                            for (ExamStudent t : examStudentList) {
                                basicAttachmentList.add(examStudentHtml(attachmentIds, studentContent, t, paperType, examCard, sysUser.getId(), examStudentPdfList));
                            }
                            examStudentService.saveOrUpdateBatch(examStudentList);
                            jsonObject.put("card", jsonArray);
                            examCardDetail.setAttachmentId(jsonObject.toJSONString());
                        }
                        examCardDetailService.saveOrUpdateBatch(examCardDetailList);
                    }
                    examTaskDetailService.saveOrUpdateBatch(examTaskDetailList);
                }
                //合并pdf
                basicAttachmentList.add(mergePdf(tbTask, examDetail, sysUser.getId(), variablePdfList, ordinaryPdfList, paperPdfList, examStudentPdfList, backupPaperPdfList, cardPdfList));
            }
            //最后一步删除附件
            deleteAttachment(attachmentIds);
        } catch (Exception e) {
            log.error("请求出错", e);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentList);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    /**
     * 创建签到表
     *
     * @param basicAttachment
     * @param examDetailCourse
     * @param schoolName
     * @param examStudentList
     * @param variablePdfList
     * @throws IOException
     */
    private void createSignBook(BasicAttachment basicAttachment,
//                                String htmlContent,
                                ExamDetailCourse examDetailCourse, String schoolName, List<ExamStudent> examStudentList, List<String> variablePdfList) throws IOException {
        /**html
         htmlContent = htmlContent.replaceAll("\\$\\{examDetailCourseId\\}", examDetailCourse.getId() + "");
         htmlContent = htmlContent.replaceAll("\\$\\{schoolName\\}", schoolName);
         htmlContent = htmlContent.replaceAll("\\$\\{courseName\\}", examDetailCourse.getCourseName());
         htmlContent = htmlContent.replaceAll("\\$\\{courseCode\\}", examDetailCourse.getCourseCode());

         //        htmlMap.put("examName", exam.getExamName());
         //        htmlMap.put("examRoom", examDetailMap.get("examRoom"));
         //        htmlMap.put("examRoomId", examDetailMap.get("examRoomId"));
         //        htmlMap.put("startTime", examDetailMap.get("startTime"));
         //        htmlMap.put("endTime", examDetailMap.get("endTime"));
         //        htmlMap.put("sceneNumberId", examDetailMap.get("sceneNumberId"));

         htmlContent = htmlContent.replaceAll("\\$\\{paperCode\\}", examDetailCourse.getPaperNumber());
         htmlContent = htmlContent.replaceAll("\\$\\{paperCodeImg\\}", GoogleBarCodeUtil.createBarCode(examDetailCourse.getPaperNumber(), false));

         //        List<Map> students = tcPExamStudentService.clientStudentByDetailId(examDetailId);

         //        List<String> extendColumnList = students.stream().map(m -> m.get("extendColumn").toString()).distinct().collect(Collectors.toList());
         //        Set<String> classTeacher = new HashSet();
         //        if (extendColumnList != null && extendColumnList.size() > 0) {
         //            for (String s : extendColumnList) {
         //                List<Map> mapList = JSONObject.parseArray(s, Map.class);
         //                for (Map map : mapList) {
         //                    if (Objects.equals("classTeacher", map.get("code").toString())) {
         //                        classTeacher.add(map.get("value").toString());
         //                    }
         //                }
         //            }
         //        }

         //        htmlMap.put("classTeacher", String.join(",", classTeacher));

         int totalCount = examStudentList.size();
         if (totalCount > 0) {
         //            List<Map> subList = new ArrayList<>();
         int pageCount = totalCount % 80 == 0 ? totalCount / 80 : totalCount / 80 + 1;
         for (int i = 0; i < pageCount; i++) {
         Map subMap = new HashMap();
         subMap.put("index", i + 1);
         int studentCount;
         List<ExamStudent> subStudents;
         if (pageCount == 1) {
         studentCount = totalCount;
         subStudents = examStudentList;
         } else if (pageCount > 1 && i == pageCount - 1) {
         studentCount = totalCount - (pageCount - 1) * 80;
         subStudents = examStudentList.subList(80 * i, examStudentList.size());
         } else {
         studentCount = 80;
         subStudents = examStudentList.subList(80 * i, 80 * (i + 1));
         }
         subMap.put("studentCount", studentCount);
         List<ExamStudentDto> examStudentList1 = new ArrayList<>();
         List<ExamStudentDto> examStudentList2 = new ArrayList<>();

         Gson gson = new Gson();
         int mod = subStudents.size() % 2;
         int htmlTableCount = mod == 0 ? subStudents.size() / 2 : subStudents.size() / 2 + 1;
         for (int j = 0; j < htmlTableCount; j++) {
         ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
         examStudentDto.setIndex(j + 1);
         examStudentList1.add(examStudentDto);
         //                    Map map = subStudents.get(j);
         //                    TcPExamStudent tcPExamStudent = new TcPExamStudent();
         //                    tcPExamStudent.setIndex(j + 1);
         //                    tcPExamStudent.setSiteNumber(String.valueOf(map.get("siteNumber")));
         //                    tcPExamStudent.setStudentCode(String.valueOf(map.get("studentCode")));
         //                    tcPExamStudent.setName(String.valueOf(map.get("name")));
         //                    tcPExamStudent.setExtendColumn(getClassName(map.get("extendColumn").toString()));
         //                    tcPExamStudentList1.add(tcPExamStudent);
         }
         for (int j = htmlTableCount; j < subStudents.size(); j++) {
         ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
         examStudentDto.setIndex(j + 1);
         examStudentList2.add(examStudentDto);
         //                    Map map = students.get(j);
         //                    TcPExamStudent tcPExamStudent = new TcPExamStudent();
         //                    tcPExamStudent.setIndex(j + 1);
         //                    tcPExamStudent.setSiteNumber(String.valueOf(map.get("siteNumber")));
         //                    tcPExamStudent.setStudentCode(String.valueOf(map.get("studentCode")));
         //                    tcPExamStudent.setName(String.valueOf(map.get("name")));
         //                    tcPExamStudent.setExtendColumn(getClassName(map.get("extendColumn").toString()));
         //                    tcPExamStudentList2.add(tcPExamStudent);
         }
         if (examStudentList1.size() > examStudentList2.size()) {
         for (int j = subStudents.size() - htmlTableCount; j < examStudentList1.size(); j++) {
         examStudentList2.add(new ExamStudentDto());
         }
         }

         //                subMap.put("studentList1", examStudentList1);
         //                subMap.put("studentList2", examStudentList2);

         //                  htmlContent = htmlContent.replaceAll("\\$\\{studentList1\\}", examStudentList1);
         //                  htmlContent = htmlContent.replaceAll("\\$\\{studentList2\\}", examStudentList2);

         //                subList.add(subMap);
         }
         //            htmlMap.put("subList", subList);
         }
         basicAttachmentService.saveAttachmentPdf(basicAttachment, variablePdfList);
         **/

        Map<String, Object> htmlMap = new HashMap<>();
        htmlMap.put("examDetailCourseId", examDetailCourse.getId());
        htmlMap.put("schoolName", schoolName);

        htmlMap.put("courseName", examDetailCourse.getCourseName());
        htmlMap.put("courseCode", examDetailCourse.getCourseCode());
//        htmlMap.put("examName", exam.getExamName());
//        htmlMap.put("examRoom", examDetailMap.get("examRoom"));
//        htmlMap.put("examRoomId", examDetailMap.get("examRoomId"));
//        htmlMap.put("startTime", examDetailMap.get("startTime"));
//        htmlMap.put("endTime", examDetailMap.get("endTime"));
//        htmlMap.put("sceneNumberId", examDetailMap.get("sceneNumberId"));
        htmlMap.put("paperCode", examDetailCourse.getPaperNumber());
        htmlMap.put("paperCodeImg", GoogleBarCodeUtil.createBarCode(examDetailCourse.getPaperNumber(), false));

//        List<String> extendColumnList = students.stream().map(m -> m.get("extendColumn").toString()).distinct().collect(Collectors.toList());
//        Set<String> classTeacher = new HashSet();
//        if (extendColumnList != null && extendColumnList.size() > 0) {
//            for (String s : extendColumnList) {
//                List<Map> mapList = JSONObject.parseArray(s, Map.class);
//                for (Map map : mapList) {
//                    if (Objects.equals("classTeacher", map.get("code").toString())) {
//                        classTeacher.add(map.get("value").toString());
//                    }
//                }
//            }
//        }
//        htmlMap.put("classTeacher", String.join(",", classTeacher));

        int totalCount = examStudentList.size();
        if (totalCount > 0) {
            List<Map> subList = new ArrayList<>();
            int pageCount = totalCount % 80 == 0 ? totalCount / 80 : totalCount / 80 + 1;
            for (int i = 0; i < pageCount; i++) {
                Map subMap = new HashMap();
                subMap.put("index", i + 1);
                int studentCount;
                List<ExamStudent> subStudents;
                if (pageCount == 1) {
                    studentCount = totalCount;
                    subStudents = examStudentList;
                } else if (pageCount > 1 && i == pageCount - 1) {
                    studentCount = totalCount - (pageCount - 1) * 80;
                    subStudents = examStudentList.subList(80 * i, examStudentList.size());
                } else {
                    studentCount = 80;
                    subStudents = examStudentList.subList(80 * i, 80 * (i + 1));
                }
                subMap.put("studentCount", studentCount);
                List<ExamStudentDto> examStudentList1 = new ArrayList<>();
                List<ExamStudentDto> examStudentList2 = new ArrayList<>();

                Gson gson = new Gson();
                int mod = subStudents.size() % 2;
                int htmlTableCount = mod == 0 ? subStudents.size() / 2 : subStudents.size() / 2 + 1;
                for (int j = 0; j < htmlTableCount; j++) {
                    ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
                    examStudentDto.setIndex(j + 1);
                    examStudentList1.add(examStudentDto);
//                    Map map = subStudents.get(j);
//                    TcPExamStudent tcPExamStudent = new TcPExamStudent();
//                    tcPExamStudent.setIndex(j + 1);
//                    tcPExamStudent.setSiteNumber(String.valueOf(map.get("siteNumber")));
//                    tcPExamStudent.setStudentCode(String.valueOf(map.get("studentCode")));
//                    tcPExamStudent.setName(String.valueOf(map.get("name")));
//                    tcPExamStudent.setExtendColumn(getClassName(map.get("extendColumn").toString()));
//                    tcPExamStudentList1.add(tcPExamStudent);
                }
                for (int j = htmlTableCount; j < subStudents.size(); j++) {
                    ExamStudentDto examStudentDto = gson.fromJson(gson.toJson(subStudents.get(j)), ExamStudentDto.class);
                    examStudentDto.setIndex(j + 1);
                    examStudentList2.add(examStudentDto);
//                    Map map = students.get(j);
//                    TcPExamStudent tcPExamStudent = new TcPExamStudent();
//                    tcPExamStudent.setIndex(j + 1);
//                    tcPExamStudent.setSiteNumber(String.valueOf(map.get("siteNumber")));
//                    tcPExamStudent.setStudentCode(String.valueOf(map.get("studentCode")));
//                    tcPExamStudent.setName(String.valueOf(map.get("name")));
//                    tcPExamStudent.setExtendColumn(getClassName(map.get("extendColumn").toString()));
//                    tcPExamStudentList2.add(tcPExamStudent);
                }
                if (examStudentList1.size() > examStudentList2.size()) {
                    for (int j = subStudents.size() - htmlTableCount; j < examStudentList1.size(); j++) {
                        examStudentList2.add(new ExamStudentDto());
                    }
                }

                subMap.put("studentList1", examStudentList1);
                subMap.put("studentList2", examStudentList2);
                subList.add(subMap);
            }
            htmlMap.put("subList", subList);
        }
        freemarkerUtil.createSignBook(htmlMap);
    }


    /**
     * 替换html通用模版
     *
     * @param examCardDetail
     * @return
     */
    public String replaceHtmlTemplete(ExamCardDetail examCardDetail) {
        String content = examCardDetail.getHtmlContent();
        log.info("contentTemp:{}", content);
        content = content.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
        content = content.replaceAll("\\$\\{examNumberStr\\}", "");
        return content;
    }

    /**
     * 获取题卡attachmentId
     *
     * @param examCardDetail
     * @param attachmentIds
     */
    public void getCardAttachmentId(ExamCardDetail examCardDetail, Set<Long> attachmentIds) {
        if (Objects.nonNull(examCardDetail.getAttachmentId())) {
            JSONObject jsonObjectCard = JSONObject.parseObject(examCardDetail.getAttachmentId());
            JSONArray jsonArrayCard = (JSONArray) jsonObjectCard.get("card");
            for (int i = 0; i < jsonArrayCard.size(); i++) {
                JSONObject object = (JSONObject) jsonArrayCard.get(i);
                if (Objects.nonNull(object.get("attachmentId"))) {
                    attachmentIds.add(Long.parseLong(String.valueOf(object.get("attachmentId"))));
                }
            }
        }
    }

    /**
     * 获取试卷pdf
     *
     * @param examTaskDetail
     * @param backupCount
     * @param pdfList
     * @return
     * @throws IOException
     */
    public void getPaperPdf(ExamTaskDetail examTaskDetail, Integer backupCount, List<String>... pdfList) throws IOException {
        JSONObject jsonObjectPaper = JSONObject.parseObject(examTaskDetail.getPaperAttachmentIds());
        JSONArray jsonArrayPaper = jsonObjectPaper.getJSONArray("paper");
        for (int i = 0; i < jsonArrayPaper.size(); i++) {
            JSONObject object = (JSONObject) jsonArrayPaper.get(i);
            if (Objects.nonNull(object.get("attachmentId"))) {
                Long attachmentId = Long.parseLong(String.valueOf(object.get("attachmentId")));
                BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                String filePath = commonService.getFile(basicAttachment.getPath(), false);
                pdfList[0].add(filePath);
                for (int j = 1; j <= backupCount; j++) {
                    pdfList[1].add(filePath);
                }
            }
        }
    }

    /**
     * 抽取试卷
     *
     * @param examPrintPlan
     * @param examTaskDetail
     * @return
     */
    public String getPaperType(ExamPrintPlan examPrintPlan, ExamTaskDetail examTaskDetail) {
        //抽取卷型
        DrawRuleEnum drawRule = Objects.nonNull(examPrintPlan.getDrawRule()) ? examPrintPlan.getDrawRule() : DrawRuleEnum.ONE;
        //未曝光卷型
        String unexposedPaperType = examTaskDetail.getUnexposedPaperType();
        //已曝光卷型
        String exposedPaperType = examTaskDetail.getExposedPaperType();
        String[] paperTypes = null;
        if (drawRule == DrawRuleEnum.ONE) {
            if (Objects.isNull(unexposedPaperType)) {
                throw ExceptionResultEnum.ERROR.exception("当前没有未曝光的卷型");
            } else {
                paperTypes = unexposedPaperType.split("/");
            }
        } else {
            if (Objects.isNull(exposedPaperType) && Objects.isNull(unexposedPaperType)) {
                throw ExceptionResultEnum.ERROR.exception("当前没有未曝光的卷型");
            }
            if (Objects.nonNull(unexposedPaperType)) {
                paperTypes = unexposedPaperType.split("/");
            } else {
                paperTypes = exposedPaperType.split("/");
            }
        }
        int paperRandom = new Random().nextInt(paperTypes.length);
        String paperType = paperTypes[paperRandom];
        examTaskDetail.setPaperType(paperType);
        return paperType;
    }

    /**
     * 合并pdf
     *
     * @param tbTask
     * @param examDetail
     * @param userId
     * @param list
     * @return
     * @throws IOException
     */
    public BasicAttachment mergePdf(TBTask tbTask, ExamDetail examDetail, Long userId, List<String>... list) throws IOException {
        StringJoiner stringJoiner = new StringJoiner("").add(SystemConstant.PDF_TEMP_FILES_DIR).add(File.separator);
        List<String> mergePdfList = new ArrayList<>();
        for (int i = 0; i < list.length; i++) {
            mergePdfList.addAll(list[i]);
        }
        String dirName = PdfUtil.mergePdf(mergePdfList.toArray(new String[mergePdfList.size()]), null);
        File localPdfFile = new File(stringJoiner.toString() + File.separator + dirName);
        BasicAttachment basicAttachment = basicAttachmentService.saveAttachmentPdf(dirName, userId);
        tbTask.setImportFileName(basicAttachment.getName());
        tbTask.setImportFilePath(basicAttachment.getPath());
        examDetail.setAttachmentId(basicAttachment.getId());
        detailService.saveOrUpdate(examDetail);
        ossUtil.ossUpload(dirName, localPdfFile, BinaryUtil.toBase64String(HexUtils.decodeHex(DigestUtils.md5Hex(new FileInputStream(localPdfFile)))));
        localPdfFile.delete();
        for (String s : mergePdfList) {
            new File(s).delete();
        }
        return basicAttachment;
    }

    /**
     * 删除附件
     *
     * @param attachmentIds
     */
    public void deleteAttachment(Set<Long> attachmentIds) {
        if (Objects.nonNull(attachmentIds) && attachmentIds.size() > 0) {
            QueryWrapper<BasicAttachment> basicAttachmentQueryWrapper = new QueryWrapper<>();
            basicAttachmentQueryWrapper.lambda().in(BasicAttachment::getId, attachmentIds);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentService.list(basicAttachmentQueryWrapper));
            basicAttachmentService.removeByIds(attachmentIds);
        }
    }

    /**
     * 考生题卡html
     *
     * @param attachmentIds
     * @param studentContent
     * @param t
     * @param paperType
     * @param examCard
     * @param userId
     * @param examStudentPdfList
     * @return
     * @throws IOException
     */
    public BasicAttachment examStudentHtml(Set<Long> attachmentIds, String studentContent, ExamStudent t, String paperType, ExamCard examCard, Long userId, List<String> examStudentPdfList) throws IOException {
        if (Objects.nonNull(t.getAttachmentId())) {
            attachmentIds.add(t.getAttachmentId());
        }
        String studentHtml = studentContent;
        if (Objects.nonNull(t.getExtendFields())) {
            JSONArray jsonObjectExtend = (JSONArray) JSONArray.parse(t.getExtendFields());//扩展字段
            if (Objects.nonNull(jsonObjectExtend)) {
                for (int i = 0; i < jsonObjectExtend.size(); i++) {
                    JSONObject object = (JSONObject) jsonObjectExtend.get(i);
                    studentHtml = studentHtml.replaceAll("\\$\\{" + object.get("code") + "\\}", String.valueOf(object.get("value")));
                }
            }
        }
        //生成学生考号条码并将图片转成base64
        studentHtml = studentHtml.replaceAll("\\$\\{examNumber\\}", GoogleBarCodeUtil.createBarCode(t.getTicketNumber(), false));
        studentHtml = studentHtml.replaceAll("\\$\\{examNumberStr\\}", t.getTicketNumber());
        //随机生成学生试卷条码并将图片转成base64
        studentHtml = studentHtml.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
        studentHtml = studentHtml.replaceAll("\\$\\{paperTypeName\\}", paperType);
        studentHtml = studentHtml.replaceAll("\\$\\{studentCode\\}", t.getStudentCode());
        studentHtml = studentHtml.replaceAll("\\$\\{name\\}", t.getStudentName());
        studentHtml = studentHtml.replaceAll("\\$\\{courseName\\}", examCard.getCourseName());

        //学生题卡
        BasicAttachment examStudentAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode() + "|" + t.getTicketNumber(), studentHtml, userId, examStudentPdfList);
        t.setAttachmentId(examStudentAttachment.getId());
        return examStudentAttachment;
    }

    /**
     * 通用题卡html
     *
     * @param cardContent
     * @param paperType
     * @param examDetailCourse
     * @param examCard
     * @param jsonArray
     * @param userId
     * @param cardPdfList
     * @return
     * @throws IOException
     */
    public BasicAttachment cardHtml(String cardContent, String paperType, ExamDetailCourse examDetailCourse, ExamCard examCard, JSONArray jsonArray, Long userId, List<String> cardPdfList) throws IOException {
        //通用题卡
        String cardTemp = cardContent;
        cardTemp = cardTemp.replaceAll("\\$\\{paperTypeName\\}", paperType);
        //随机生成试卷条码并将图片转成base64
        cardTemp = cardTemp.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
        //通用题卡生成卷袋贴条码
        String paperCode = examDetailCourse.getPaperNumber();
        String paperCodeImg = GoogleBarCodeUtil.createBarCode(paperCode, false);
        String paperCodeDiv = "<div class=\"page-box page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 80px;right: 35px;transform: rotate(-90deg);transform-origin: center right;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + paperCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + paperCode + "</p></div>";
        cardTemp = cardTemp.replaceAll("<div class=\"page-box page-box-0\">", paperCodeDiv);
        BasicAttachment cardAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode(), cardTemp, userId, cardPdfList);
        JSONObject object = new JSONObject();
        object.put("name", paperType);
        object.put("examDetailCourseId", examDetailCourse.getId());
        object.put("attachmentId", cardAttachment.getId());
        jsonArray.add(object);
        return cardAttachment;
    }
}
