package com.qmth.distributed.print.business.util;

import com.qmth.distributed.print.business.config.DictionaryConfig;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import javax.annotation.Resource;
import java.io.*;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: freemarker util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/11
 */
@Component
public class FreemarkerUtil {
    private final static Logger log = LoggerFactory.getLogger(FreemarkerUtil.class);

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    OssUtil ossUtil;

    /**
     * 单独生成签到表
     *
     * @param dataMap
     * @throws IOException
     */
    public void createSignBook(Map<String, Object> dataMap) throws IOException {
        if (Objects.isNull(dataMap) || dataMap.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("签到表数据为空");
        }
        Configuration configuration = new Configuration(Configuration.VERSION_2_3_29);
        if (SystemConstant.TEMP_FILES_DIR.startsWith(ResourceUtils.CLASSPATH_URL_PREFIX)) {
            File templates = ResourceUtils.getFile(SystemConstant.TEMP_FILES_DIR);
            configuration.setDirectoryForTemplateLoading(templates);
        } else {
//            templates = new File(dictionaryConfig.sysDomain().getFreemarkerPath());
            configuration.setClassForTemplateLoading(this.getClass(), SystemConstant.TEMP_FILES_DIR);//classes下读取
        }
        this.createSignBookAndPaperPackageHtml(configuration, "signBook", (String) dataMap.get("schoolName"), dataMap);
    }

    /**
     * 单独生成签到表
     *
     * @param dataMap
     * @throws IOException
     */
    public void createPaperPackage(Map<String, Object> dataMap) throws IOException {
        if (Objects.isNull(dataMap) || dataMap.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("卷袋贴数据为空");
        }
        Configuration configuration = new Configuration(Configuration.VERSION_2_3_29);
        if (SystemConstant.TEMP_FILES_DIR.startsWith(ResourceUtils.CLASSPATH_URL_PREFIX)) {
            File templates = ResourceUtils.getFile(SystemConstant.TEMP_FILES_DIR);
            configuration.setDirectoryForTemplateLoading(templates);
        } else {
//            templates = new File(dictionaryConfig.sysDomain().getFreemarkerPath());
            configuration.setClassForTemplateLoading(this.getClass(), SystemConstant.TEMP_FILES_DIR);//classes下读取
        }
        this.createSignBookAndPaperPackageHtml(configuration, "paperPackage", (String) dataMap.get("schoolName"), dataMap);
    }

    /**
     * 创建签到表
     *
     * @param configuration
     * @param schoolName
     * @param dataMap
     * @throws IOException
     */
    private void createSignBookAndPaperPackageHtml(Configuration configuration, String ftlName, String schoolName, Map<String, Object> dataMap) throws IOException {
        Writer out = null;
        try {
            // step1 加载模版文件
            Template template = configuration.getTemplate(ftlName + ".ftl");
            // step2 生成数据
            String serverUpload = dictionaryConfig.sysDomain().getServerUpload();
            File mkdir = null;
            if (Objects.nonNull(serverUpload) && !Objects.equals(serverUpload, "")) {
                mkdir = new File(serverUpload + File.separator + UploadFileEnum.HTML.name());
                if (!mkdir.exists()) {
                    mkdir.mkdirs();
                }
            }
            File htmlFile = new File(new StringJoiner("").add(mkdir.toString()).add(File.separator).add(schoolName).add(ftlName).add(".html").toString());
            out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(htmlFile)));
            // step3 输出文件
            template.process(dataMap, out);
            out.flush();
            boolean oss = dictionaryConfig.sysDomain().isOss();
            if (oss) {
                StringJoiner stringJoiner = new StringJoiner("").add(UploadFileEnum.HTML.name())
                        .add(File.separator).add(schoolName).add(htmlFile.getName());
                ossUtil.ossUpload(stringJoiner.toString(), new FileInputStream(htmlFile), DigestUtils.md5Hex(new FileInputStream(htmlFile)));
            }
            log.info("{}文件创建成功", htmlFile.toString());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(out)) {
                out.close();
            }
        }
    }
}
